@@ -1,5 +1,3 @@
-from typing import NewType
-
 import torch
 
 from lumo.core import Params
@@ -8,14 +6,15 @@ from .factory import OptimFactory, InterpFactory
 
 
 class TrainerExperiment(SimpleExperiment):
+    """A class for helping manage an experiment by Trainer."""
 
     @property
     def log_dir(self):
-        return self.test_root
+        return self.info_dir
 
     @property
     def params_fn(self):
-        res = self.test_file('params.yaml')
+        res = self.mk_ipath('params.yaml')
         self.dump_string('params.yaml', res)
         return res
 
@@ -25,7 +24,7 @@ class TrainerExperiment(SimpleExperiment):
         if self.has_prop(key):
             return self.get_prop(key)
         else:
-            log_dir = self.test_dir('board')
+            log_dir = self.mk_ipath('board', is_dir=True)
             res = {
                 'filename_suffix': '.bd',
                 'log_dir': log_dir,
@@ -35,23 +34,24 @@ class TrainerExperiment(SimpleExperiment):
 
     @property
     def state_dict_dir(self):
-        res = self.blob_dir('state_dict')
+        res = self.mk_bpath('state_dict', is_dir=True)
         return res
 
     def dump_train_eidx(self, eidx, epoch: int):
         """
-        Args:
-            eidx: start from 0, end at `epoch-1`
-            epoch:
+        Dumps the progress of the trainer.
+
+         Args:
+            eidx (int): The index of the current epoch (starting from 0).
+            epoch (int): The total number of epochs to train for.
         """
         self.dump_progress((eidx + 1) / epoch, update_from='trainer')
 
 
-class ReimplementExperiment(TrainerExperiment):
-    pass
-
-
 class TrainerParams(Params):
+    """
+    A class to hold parameters for trainer.
+    """
     OPTIM = OptimFactory
     SCHE = INTERP = InterpFactory
 
