@@ -3,6 +3,17 @@ import os
 
 
 def analyse_module_dependency(module, mem=None, root=None):
+    """
+    Recursively analyse the dependencies of a module and return a dictionary of modules and their associated files.
+
+    Args:
+        module (module): The module to analyse.
+        mem (dict, optional): A dictionary of previously analysed modules and their associated files.
+        root (str, optional): The root directory to use as a reference when determining whether a file is a dependency.
+
+    Returns:
+        dict: A dictionary of modules and their associated files.
+    """
     if mem is None:
         mem = {}
 
